# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json
import base64
import requests
from .settings import (MARTOR_IMGUR_CLIENT_ID, MARTOR_IMGUR_API_KEY)

requests.packages.urllib3.disable_warnings()


def imgur_uploader(image):
    """
    Basic imgur uploader return as json data.
    :param `image` is from `request.FILES['markdown-image-upload']`
    :return json response
    """
    api_url = 'https://api.imgur.com/3/upload.json'
    headers = {'Authorization': 'Client-ID ' + MARTOR_IMGUR_CLIENT_ID}
    response = requests.post(
        api_url,
        headers=headers,
        data={
            'key': MARTOR_IMGUR_API_KEY,
            'image': base64.b64encode(image.read()),
            'type': 'base64',
            'name': image.name
        }
    )

    if response.status_code == 200:
        response_data = json.loads(response.content.decode('utf-8'))
        return json.dumps({
            'status': response_data['status'],
            'link': response_data['data']['link'],
            'name': response_data['data']['name']
        })

    elif response.status_code == 415:
        # Unsupport File type
        return json.dumps({
            'status': response.status_code,
            'error': response.reason
        })

    return json.dumps({
        'status': response.status_code,
        'error': response.content.decode('utf-8')
    })



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.http import (HttpResponse, JsonResponse)
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.decorators import login_required
from django.contrib.auth import get_user_model
from django.db.models import Q

from .api import imgur_uploader
from .settings import MARTOR_MARKDOWNIFY_FUNCTION
from .utils import LazyEncoder

User = get_user_model()


def markdownfy_view(request):
    if request.method == 'POST':
        content = request.POST.get('content', '')
        markdownify = import_string(MARTOR_MARKDOWNIFY_FUNCTION)
        return HttpResponse(markdownify(content))
    return HttpResponse(_('Invalid request!'))


@login_required
def markdown_imgur_uploader(request):
    """
    Makdown image upload for uploading to imgur.com
    and represent as json to markdown editor.
    """
    if request.method == 'POST' and request.is_ajax():
        if 'markdown-image-upload' in request.FILES:
            image = request.FILES['markdown-image-upload']
            response_data = imgur_uploader(image=image)
            return HttpResponse(response_data, content_type='application/json')
        return HttpResponse(_('Invalid request!'))
    return HttpResponse(_('Invalid request!'))


@login_required
def markdown_search_user(request):
    """
    Json usernames of the users registered & actived.

    url(method=get):
        /martor/search-user/?username={username}

    Response:
        error:
            - `status` is status code (204)
            - `error` is error message.
        success:
            - `status` is status code (204)
            - `data` is list dict of usernames.
                { 'status': 200,
                  'data': [
                    {'usernane': 'john'},
                    {'usernane': 'albert'}]
                }
    """
    response_data = {}
    username = request.GET.get('username')

    if username is not None \
            and username != '' \
            and ' ' not in username:

        users = User.objects.filter(
            Q(username__iexact=username) |
            Q(username__icontains=username)
        ).filter(is_active=True)

        if users.exists():
            response_data.update({'status': 200,
                                  'data': [{'username': u.username} for u in users]})
            return JsonResponse(response_data)

        response_data.update({'status': 204,
                              'error': _('No users registered as `%(username)s` '
                                         'or user is unactived.') % {'username': username}})
    else:
        response_data.update({'status': 204,
                              'error': _('Validation Failed for field `username`')})

    return JsonResponse(response_data)




import django

from .views import TestFormView


if django.VERSION >= (2, 0):
    from django.urls import path, include
    urlpatterns = [
        path('test-form-view/', TestFormView.as_view()),
        path('martor/', include('martor.urls')),
    ]
else:
    from django.conf.urls import url, include
    urlpatterns = [
        url(r'^test-form-view/$', TestFormView.as_view()),
        url(r'^martor/', include('martor.urls')),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms

from .settings import MARTOR_ENABLE_LABEL
from .widgets import (MartorWidget, AdminMartorWidget)


class MartorFormField(forms.CharField):

    def __init__(self, *args, **kwargs):

        # to setup the editor without label
        if not MARTOR_ENABLE_LABEL:
            kwargs['label'] = ''

        super(MartorFormField, self).__init__(*args, **kwargs)

        if not issubclass(self.widget.__class__, MartorWidget):
            self.widget = MartorWidget()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms
from django.template.loader import get_template
from django.contrib.admin import widgets

from .settings import (
    MARTOR_THEME,
    MARTOR_ENABLE_CONFIGS,
    MARTOR_UPLOAD_URL,
    MARTOR_MARKDOWNIFY_URL,
    MARTOR_SEARCH_USERS_URL,
    MARTOR_MARKDOWN_BASE_EMOJI_URL,
    MARTOR_TOOLBAR_BUTTONS,
    MARTOR_ALTERNATIVE_JS_FILE_THEME,
    MARTOR_ALTERNATIVE_CSS_FILE_THEME,
    MARTOR_ALTERNATIVE_JQUERY_JS_FILE
)


def get_theme():
    """ function to get the selected theme """
    supported_themes = ['bootstrap', 'semantic']
    if MARTOR_THEME in supported_themes:
        return MARTOR_THEME
    return 'bootstrap'


class MartorWidget(forms.Textarea):

    def render(self, name, value, attrs=None, renderer=None, **kwargs):
        # Make the settings the default attributes to pass
        attributes_to_pass = {
            'data-enable-configs': MARTOR_ENABLE_CONFIGS,
            'data-upload-url': MARTOR_UPLOAD_URL,
            'data-markdownfy-url': MARTOR_MARKDOWNIFY_URL,
            'data-search-users-url': MARTOR_SEARCH_USERS_URL,
            'data-base-emoji-url': MARTOR_MARKDOWN_BASE_EMOJI_URL
        }

        # Make sure that the martor value is in the class attr passed in
        if 'class' in attrs:
            attrs['class'] += ' martor'
        else:
            attrs['class'] = 'martor'

        # Update and overwrite with the attributes passed in
        attributes_to_pass.update(attrs)

        # Update and overwrite with any attributes that are on the widget
        # itself. This is also the only way we can push something in without
        # being part of the render chain.
        attributes_to_pass.update(self.attrs)

        template = get_template('martor/%s/editor.html' % get_theme())
        emoji_enabled = MARTOR_ENABLE_CONFIGS.get('emoji') == 'true'
        mentions_enabled = MARTOR_ENABLE_CONFIGS.get('mention') == 'true'

        widget = super(MartorWidget, self).render(name, value, attributes_to_pass)

        return template.render({
            'martor': widget,
            'field_name': name,
            'emoji_enabled': emoji_enabled,
            'mentions_enabled': mentions_enabled,
            'toolbar_buttons': MARTOR_TOOLBAR_BUTTONS
        })

    class Media:
        selected_theme = get_theme()
        css = {
            'all': (
                'plugins/css/ace.min.css',
                'plugins/css/resizable.min.css',
                'martor/css/martor.%s.css' % selected_theme,
                'martor/css/martor-admin.css'
            )
        }
        js = (
            'plugins/js/ace.js',
            'plugins/js/mode-markdown.js',
            'plugins/js/ext-language_tools.js',
            'plugins/js/theme-github.js',
            'plugins/js/highlight.min.js',
            'plugins/js/resizable.min.js',
            'plugins/js/emojis.min.js',
            'martor/js/martor.%s.js' % selected_theme,
        )

        # Adding the following scripts to the end of the tuple in case it affects behaviour.
        # spellcheck configuration
        if MARTOR_ENABLE_CONFIGS.get('spellcheck') == 'true':
            js = ('plugins/js/typo.js', 'plugins/js/spellcheck.js').__add__(js)

        # support alternative vendor theme file like: bootstrap, semantic)
        # 1. vendor css theme
        if MARTOR_ALTERNATIVE_CSS_FILE_THEME:
            css_theme = MARTOR_ALTERNATIVE_CSS_FILE_THEME
            css['all'] = (css_theme,).__add__(css.get('all'))
        else:
            css_theme = 'plugins/css/%s.min.css' % selected_theme
            css['all'] = (css_theme,).__add__(css.get('all'))

        # 2. vendor js theme
        if MARTOR_ALTERNATIVE_JS_FILE_THEME:
            js_theme = MARTOR_ALTERNATIVE_JS_FILE_THEME
            js = (MARTOR_ALTERNATIVE_JS_FILE_THEME,).__add__(js)
        else:
            js_theme = 'plugins/js/%s.min.js' % selected_theme
            js = (js_theme,).__add__(js)

        # 3. vendor jQUery
        if MARTOR_ALTERNATIVE_JQUERY_JS_FILE:
            js = (MARTOR_ALTERNATIVE_JQUERY_JS_FILE,).__add__(js)
        elif MARTOR_ENABLE_CONFIGS.get('jquery') == 'true':
            js = ('plugins/js/jquery.min.js',).__add__(js)


class AdminMartorWidget(MartorWidget, widgets.AdminTextareaWidget):
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models

from .fields import MartorFormField


class MartorField(models.TextField):

    def formfield(self, **kwargs):
        defaults = {'form_class': MartorFormField}
        defaults.update(kwargs)
        return super(MartorField, self).formfield(**defaults)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import django
from .views import (markdownfy_view,
                    markdown_imgur_uploader,
                    markdown_search_user)


if django.VERSION >= (2, 0):
    from django.urls import path
    urlpatterns = [
        path('markdownify/', markdownfy_view, name='martor_markdownfy'),
        path('uploader/', markdown_imgur_uploader, name='imgur_uploader'),
        path('search-user/', markdown_search_user, name='search_user_json'),
    ]
else:
    from django.conf.urls import url
    urlpatterns = [
        url(r'^markdownify/$', markdownfy_view, name='martor_markdownfy'),
        url(r'^uploader/$', markdown_imgur_uploader, name='imgur_uploader'),
        url(r'^search-user/$', markdown_search_user, name='search_user_json'),
    ]
